##----------------------------------------------------------------
## Init by deleting all variables and functions
rm(list=ls())
## Set the working directory
setwd(".")

## Source the scripts with functions in the "functions" and "models" folder
files <- dir("functions", full.names=TRUE)
for(i in 1:length(files)) source(files[i])

## Use the ctsmr package
library(ctsmr)

## Read the data into a data.frame
X <- read.csv("inputPRBS1.csv",sep=";",header=TRUE)
X$timedate <- asP("2009-02-05 14:26:00)") + X$t * 3600
##----------------------------------------------------------------


##----------------------------------------------------------------
## The simple model and the one step extended models

## Run the parameter optimization for the first model
fitTi <- Ti(X)
## Analyze the results (If you want in RStudio not to have external plot windows, change "newdev", maybe the default value in "functions/analyzeFit.R")
analyzeFit(fitTi, newdev=TRUE)
##----------------------------------------------------------------


##----------------------------------------------------------------
## Run the parameter optimization
fitTiTe <- TiTe(X)
## Analyze the results
analyzeFit(fitTiTe)
##----------------------------------------------------------------


##----------------------------------------------------------------
##
fitTi$loglik
fitTiTe$loglik


## Perform a likelihood ratio test: lambda = lik(smallerModel)/lik(largerModel) ,
## where the smallerModel is submodel of the largerModel and lambda is chi2(f)
## distributed with f=dim(smallerModel)-dim(largerModel). Page 20 in Madsen2006.

## Take the results of both models
small <- fitTi
large <- fitTiTh
## Calculate the logLikelihood for both models from their fit
logLikSmallModel <- small$loglik
logLikLargeModel <- large$loglik
## Calculate lambda which follows the chisquare distribution
chisqStat <- -2 * (logLikSmallModel - logLikLargeModel)
## It this gives a p-value smaller than confidence limit, i.e. 5\%, then the
## larger model is significant better than the smaller model
prmDiff <- large$model$NPARAM - small$model$NPARAM
## The p-value of the test, if it is <0.05 the larger model should be preferred over the smaller
1 - pchisq(chisqStat, prmDiff)
##----------------------------------------------------------------

##----------------------------------------------------------------
## liklihood ratio test wrapped in a function
likRatioTest(fitTiTh, fitTi)
##----------------------------------------------------------------
