ctsmr.stochastic.simulate2 <- function(object, nsim = 1L, seed = NULL, newdata = NULL, firstorderinputinterpolation = FALSE, covariance = FALSE, x0 = NULL, vx0 = NULL, nsub=10, printi=FALSE, ...) 
{
  ## Inputs:
  ## object (model)
  ## nsim = 1L (Not used)
  ## seed = NULL (Not used)
  ## newdata = NULL
  ## firstorderinputinterpolation = FALSE 
  ## covariance = FALSE (Not used)
  ## x0 = NULL (initial values)
  ## vx0 = NULL (initial variance)
  ## nsub (Number of subsamples between timepoints)
  dt <- diff(newdata$t[1:2])/nsub
  ## ... (Not used)
  ##
  ##
  ctsmr:::ValidateResultObject(object)
  if (missing(newdata) || is.null(newdata)) {
    newdata <- object$data
    firstorderinputinterpolation <- attr(object$data, "FirstOrderInputInterpolation")
  }
  m <- object$model
  ##
  outputsize <- 2L * (m$N + m$S) + ifelse(covariance, m$N * 
                                                      (m$N - 1L)/2L + m$S * (m$S - 1L)/2L, 0L)
  ## Subsampled data set
  datasub <- data.frame(t=seq(newdata$t[1], newdata$t[length(newdata$t)], by=dt))
  ##
  ## Interpolate
  for(i in 2:ncol(newdata)){
    datasub[ ,names(newdata)[i]] <- approx(x=newdata$t, y=newdata[ ,i], xout=datasub$t, method=c("constant","linear")[firstorderinputinterpolation+1])$y
  }
  ## To keep the output in
  statesMat <- matrix(NA, nrow=nrow(datasub), ncol=length(m$states))
  statesMat[1, ] <- x0
  colnames(statesMat) <- names(x0)
  ##
  outMat <- matrix(NA, nrow=nrow(datasub), ncol=length(m$outputs))
  colnames(outMat) <- m$outputs
  e <- ctsmr:::PrepareEnv(object, datasub[1:2, ], firstorderinputinterpolation = firstorderinputinterpolation, 
                          job = -1L, outputsize = outputsize, covariance = ifelse(covariance, 3L, 0L), x0 = x0, vx0 = vx0)
  ##
  PV.tmp <- data.frame(initial = object$xm)
  ## Run for all the subsamples
  for(i in 2:nrow(datasub)){
    if(printi){ print(pst("i=",i," of ",nrow(datasub))) }
    ##
    ## Modify only the data in the environment, see inside ctsmr:::PrepareEnv
    ## The initial values
    x0 <- statesMat[i-1, ]
    PV.tmp[names(x0), "initial"] <- x0
    doubls <- ctsmr:::genDoubles(PV.tmp, m, pred = TRUE)
    assign(".doubls", as.double(doubls), e)
    ##
    datatmp <- ctsmr:::ValidateData(datasub[(i-1):i, ], m$inputs, m$outputs, firstorderinputinterpolation)
    imat <- ctsmr:::PackData(datatmp, m$inputs, m$outputs)
    assign(".tmat", as.double(attr(imat, "time")), e)
    assign(".imat", as.double(imat), e)
    ##
    ctsmr:::CallFunction("ctsmllike", m, e)
    ##
    if ((info <- get(".info", e)) != 0) 
      stop(infocode(info))
    omat <- matrix(get(".omat", e), ncol = outputsize, byrow = TRUE)
    ## New states and output
    statesMat[i, ] <- omat[2, 1L:m$N]
    cpos <- m$N * ifelse(covariance, (m$N + 1L)/2L, 1L)
    cpos <- cpos + m$N
    outMat[i, ] <- omat[2, cpos + seq.int(1L, m$S)]
    ## Add system noise
    for(ii in 1:length(m$states)){
      stddev <- eval(m$sys.eqs[[ii]]$parsed[[2]], as.list(fit$xm))
      statesMat[i,ii] <- statesMat[i,ii] + stddev*rnorm(1, sd=sqrt(dt))
    }
  }
  ##
  ## Get out only the values at t
  i <- (0:(nrow(newdata)-1))*nsub + 1
  x <- as.data.frame(statesMat[i, ])
  names(x) <- colnames(statesMat)
  ## The output and add measurement noise
  y <- as.data.frame(outMat[i, ])
  names(y) <- colnames(outMat)
  for(ii in 1:length(m$outputs)){
    y[ ,ii] <- y[ ,ii] + rnorm(nrow(y), sd=sqrt(eval(m$variance[[ii]]$var, as.list(fit$xm))))
  }
  ## Return
  list(state = x, output = y)  
}




## stochastic.simulate1 <- function(fit, x0, data, nsub) 
## {
##   ## fit
##   ## x0 <- fit$xm[pst(model$states,"0")]
##   ## data <- D
##   ## nsub <- 10
##   ##
##   ## Calc dt
##   dt <- diff(data$t[1:2])/nsub
##   ##
##   if (!is.data.frame(data)) {
##         stop("data must be a data.frame")
##     }
##     m <- fit$model
##     if (!all(m$inputs %in% colnames(data))) {
##         stop("All inputs must be available in data")
##     }
##     time <- c("t", "time")
##     timei <- c("t", "time") %in% colnames(data)
##     if (all(timei)) {
##         stop("The time must be called either t or time.")
##     }
## #    t <- data[, time[timei]]
## #    uo <- c(t(as.matrix(data[, m$inputs])))
## #    nt <- nrow(data)
## #    x <- matrix(0, nrow = nt, ncol = m$N)
## #    y <- matrix(0, nrow = nt, ncol = m$S)
##   ##
##   datasub <- data.frame(t=seq(data$t[1], data$t[length(data$t)], by=dt))
##   ##
##   ## Linear interpolate
##   for(i in 2:ncol(data)){
##     datasub[ ,names(data)[i]] <- approx(data$t, data[ ,i], datasub$t)$y
##   }
##   ## To keep the output in
##   statesMat <- matrix(NA, nrow=nrow(datasub), ncol=length(m$states))
##   statesMat[1, ] <- x0
##   colnames(statesMat) <- names(x0)
##   ##
##   outMat <- matrix(NA, nrow=nrow(datasub), ncol=length(m$outputs))
##   ##
##   ## Run for all the subsamples
##   for(i in 2:nrow(datasub)){  
##     tmp <- simulate(fit, newdata=datasub[(i-1):(i), ], x0=statesMat[i-1, ])
##     ## The new state
##     statesMat[i, ] <- tmp$state$sim[2, ]
##     ## The new output
##     outMat[i, ] <- tmp$output$sim[2, ]
##     ## Add system noise
##     for(ii in 1:length(m$states)){
##       stddev <- eval(m$sys.eqs$X$parsed$dw1, as.list(fit$xm))
##       statesMat[i,ii] <- statesMat[i,ii] + stddev*rnorm(1, sd=sqrt(dt))
##     }
##   }
##   ##
##   ## Get out only the values at t
##   i <- (0:(nrow(data)-1))*nsub + 1
##   x <- statesMat[i, ]
##   ## The output and add measurement noise
##   y <- outMat[i, ]
##   y <- y + rnorm(length(y), sd=sqrt(eval(m$variance[[1]]$var, as.list(fit$xm))))
##   ## Return
##   list(state = x, output = y)  
## }
