# Load the functions in the "functions" folder
sapply(dir("functions/", full.names = TRUE), source)

par(mar=c(2, 4, 0.5, 1.5))
## Plot
x <- rnorm(141)
plot(x, type="n", xlab="Time", ylab="")
points(x)
lines(x, type='h')

## Autocorrelation function
acf(x)

## Autocorrelation function
acf(x)

par(mar=c(2, 4, 0.5, 1.5))
## Plot
x <- filter(rnorm(141), 0.9, "recursive")
plot(x, type="n", xlab="Time", ylab="")
points(x)
lines(x, type='h')

## Autocorrelation function
acf(x)

## Autocorrelation function
acf(x)

require(ctsmr)
## Generate a new object of class ctsm
model <- ctsm$new()
## Add a system equation and thereby also a state
model$addSystem(dTi ~ ( 1/(Ci*Ria)*(Te-Ti) )*dt + exp(p11)*dw1)
## Set the names of the inputs
model$addInput(Te)
## Set the observation equation: Ti is the state, yTi is the measured output
model$addObs(yTi ~ Ti)
## Set the variance of the measurement error
model$setVariance(yTi ~ exp(e11))
##
tmp <- read.csv("inputPRBS1.csv",sep=";",header=TRUE)
tmp$timedate <- asP("2009-02-05 14:26:00)") + tmp$t * 3600
##
names(tmp)[2] <- "Te"
##
X <- resampleDF(tmp,1,0)#[1:nrow(X),"Te"]

## Set the initial value (for the optimization) of the value of the state at the starting   time point
model$setParameter(  Ti = c(init=X$Te[1]  ,lb=0     ,ub=25 ) )
## Set the initial value for the optimization
model$setParameter(  Ci = c(init=3   ,lb=1E-5  ,ub=20 ) )
model$setParameter( Ria = c(init=5  ,lb=1     ,ub=1E4) )
model$setParameter( p11 = c(init=0.01   ,lb=-30   ,ub=10 ) )
model$setParameter( e11 = c(init=0.01  ,lb=-50   ,ub=10 ) )
## Needs to be runned for simulate to work
model$AnalyseModel()
## Make a fit object to pass to simulate.ctsmr
fit <- list()
fit$model <- model
fit$xm <- c(model$ParameterValues$initial)
names(fit$xm) <- row.names(model$ParameterValues)
## Simulate with the initial values
tmp <- X
tmp$Te + rnorm(nrow(tmp),sd=0.1)
sim <- ctsmr:::simulate.ctsmr(fit, newdata=X)
## Add to data
set.seed(873)
X$yTi <- sim$output$sim$yTi + rnorm(nrow(X),sd=0.1)
X$yTiNoNoise <- sim$output$sim$yTi
X$yTi.l1 <- lagVec(X$yTi,1)
X$Te.l1 <- lagVec(X$Te,1)
## Plot
plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
lines(X$timedate, X$yTiNoNoise,col=2)
legend("topright",c("$T\\n{e}$","$T\\n{i}$"),lty=1,col=1:2)

## Plot
plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
title(main="Measurement noise added on $T_{\\mathrm{i}}$", line=-1.5)
lines(X$timedate, X$yTi,col=2)
legend("topright",c("$T^{\\n{e}}_{t}$","$T^{\\n{i}}_{t}$"),lty=1,col=1:2)

##
plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
lines(X$timedate, X$yTi, col=2)
##
fit <- lm(yTi~0+Te.l1,X)
summary(fit)
lines(X$timedate,predict(fit,newdata=X),col=4)  
legend("topright",c("$T^{\\n{e}}_{t}$","$T^{\\n{i}}_{t}$","Predicted"),lty=1,col=c(1:2,4))

##
plot(X$timedate[-1], fit$residuals, type="n", xlab="Time", ylab="$\\hat{\\varepsilon}_t$")
abline(h=0, col="grey75")
points(X$timedate[-1], fit$residuals)
lines(X$timedate[-1], fit$residuals, type='h')

X$residuals <- NA
X$residuals[as.integer(names(fit$residuals))] <- fit$residuals
X$Te <- X$Te
acf(X$residuals, main="", na.action=na.pass)

  ##
  fit2 <- lm(yTi ~ 0 + Te.l1 + yTi.l1, X)
  summary(fit2)
  ##
  plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
  lines(X$timedate, X$yTi, col=2)
  ##
  lines(X$timedate, predict(fit2,newdata=X), col=4)  
  legend("topright",c("$T\\n{e}$","$T\\n{i}$","Predicted"),lty=1,col=c(1:2,4))

plot(X$timedate,X$yTi-predict(fit2,newdata=X),type="n",xlab="Time",ylab="$\\hat{\\varepsilon}_t$")
abline(h=0,col="grey75")
points(X$timedate,X$yTi-predict(fit2,newdata=X))
points(X$timedate,X$yTi-predict(fit2,newdata=X), type="h")

acf(X$yTi-predict(fit2,newdata=X),na.action=na.pass,main="")

require('marima')
names(X)
class(X)
Dat <- X[ ,c('Te','yTi')]
set.seed(298)
Dat$NAout <- Dat$yTi + rnorm(nrow(Dat))
Mod <- define.model(kvar=3, ar=1, ma=1, reg.var=c(1), indep=3, no.dep=c(2,3))
arp <- Mod$ar.pattern
map <- Mod$ma.pattern
## Print out model in 'short form':
short.form(arp)
short.form(map)
## Now call marima:
fit <- marima(t(Dat), ar.pattern=arp, ma.pattern=map, penalty=0.0)
(ar.model<-fit$ar.estimates)
(ma.model<-fit$ma.estimates)
X$residuals_armax <- fit$residuals[2,]
##
plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
lines(X$timedate, X$yTi, col=2)
##
lines(X$timedate, X$yTi-X$residuals_armax, col=4)  
legend("topright",c("$T\\n{e}$","$T\\n{i}$","Predicted"),lty=1,col=c(1:2,4))

acf(X$residuals_armax, main='', na.action=na.pass)

## Generate a new object of class ctsm
model <- ctsm$new()
## Add a system equation and thereby also a state
model$addSystem(dTi ~ ( 1/tau*(Te-Ti) )*dt + exp(p11)*dw1)
## Set the names of the inputs
model$addInput(Te)
## Set the observation equation: Ti is the state, yTi is the measured output
model$addObs(yTi ~ Ti)
## Set the variance of the measurement error
model$setVariance(yTi ~ exp(e11))


## Set the initial value (for the optimization) of the value of the state at the starting time point
model$setParameter(  Ti = c(init=5     ,lb=-5  ,ub=20 ) )
## Set the initial value for the optimization
model$setParameter( tau = c(init=10   ,lb=1E-2 ,ub=200 ) )
model$setParameter( p11 = c(init=0.01 ,lb=-30  ,ub=10 ) )
model$setParameter( e11 = c(init=0.01 ,lb=-50  ,ub=10 ) )

fit <- model$estimate(X)

# Teke the one-step predictions from the fit
val <- predict(fit)[[1]]
# Calculate the residuals
residualsgb <- unlist(X$yTi - val$output$pred)

# The autocorrelation function
acf(residualsgb)

  plot(X$timedate, X$Te, type="s", xlab="Time", ylab="Temperature")
  lines(X$timedate, X$yTi, col=2)
  ##
  lines(X$timedate, X$yTi-X$residuals_armax, col=4)  
  lines(X$timedate, X$yTi-residualsgb, col=5)  
  legend("topright",c("$T\\n{e}$","$T\\n{i}$","Predicted ARMAX","Predicted grey-box"),lty=1,col=c(1:2,4:5))

plot(X$timedate, X$residuals_armax, xlab="Time", ylab="Residuals", type="b", ylim=c(-0.45,0.45))
lines(X$timedate, residualsgb, type="b", col=2)
legend("topright", c("ARMAX","GB"), lty=1, col=1:2)

